@@ -5,20 +5,21 @@ from libifstate.routing import Tables, Rules
 from libifstate.sysctl import Sysctl
 from libifstate.parser import Parser
 from libifstate.tc import TC
+from libifstate.exception import netlinkerror_classes
 
 from pyroute2.netlink.rtnl.ifaddrmsg import IFA_F_PERMANENT
-from pyroute2.netlink.exceptions import NetlinkError
 try:
     from libifstate.wireguard import WireGuard
 except ModuleNotFoundError:
     # ignore missing plugin
     pass
-except NetlinkError:
+except Exception as err:
     # ignore plugin failure if kmod is not loaded
-    pass
+    if not isinstance(err, netlinkerror_classes):
+        raise
 
 from libifstate.util import logger, ipr, IfStateLogging
-from libifstate.exception import FeatureMissingError, LinkCircularLinked, LinkNoConfigFound, NetlinkError, ParserValidationError
+from libifstate.exception import FeatureMissingError, LinkCircularLinked, LinkNoConfigFound, ParserValidationError
 from ipaddress import ip_network, ip_interface
 from jsonschema import validate, ValidationError
 from copy import deepcopy
@@ -28,7 +29,7 @@ import re
 import json
 import errno
 
-__version__ = "1.5.6"
+__version__ = "1.5.7"
 
 
 class IfState():
@@ -122,6 +123,64 @@ class IfState():
             if 'sysctl' in ifstate:
                 self.sysctl.add(name, ifstate['sysctl'])
 
+            if 'cshaper' in ifstate:
+                profile_name = ifstate['cshaper'].get(
+                    'profile', 'default')
+                logger.debug('cshaper profile {} enabled'.format(profile_name),
+                             extra={'iface': name})
+                cshaper_profile = ifstates['cshaper'][profile_name]
+
+                # ingress
+                ifb_name = re.sub(
+                    cshaper_profile['ingress_ifname']['search'], cshaper_profile['ingress_ifname']['replace'], name)
+                logger.debug('cshaper ifb name {}'.format(ifb_name),
+                             extra={'iface': name})
+
+                ifb_state = {
+                    'name': ifb_name,
+                    'link': {
+                        'state': 'up',
+                        'kind': 'ifb',
+                    },
+                    'tc': {
+                        'qdisc': cshaper_profile['ingress_qdisc'],
+                    }
+                }
+                ifb_state['tc']['qdisc']['bandwidth'] = ifstate['cshaper'].get(
+                    'ingress', 'unlimited')
+
+                ifstates['interfaces'].append(ifb_state)
+
+                # egress
+                if 'tc' in ifstate:
+                    logger.warning(
+                        'cshaper settings replaces tc settings', extra={'iface': name})
+
+                ifstate['tc'] = {
+                    'ingress': True,
+                    'qdisc': cshaper_profile['egress_qdisc'],
+                    'filter': [
+                        {
+                            'kind': 'matchall',
+                            'parent': 'ffff:',
+                            'action': [
+                                {
+                                    'kind': 'mirred',
+                                    'direction': 'egress',
+                                    'action': 'redirect',
+                                    'dev': ifb_name,
+                                }
+                            ]
+                        }
+
+                    ]
+                }
+
+                ifstate['tc']['qdisc']['bandwidth'] = ifstate['cshaper'].get(
+                    'egress', 'unlimited')
+
+                del ifstate['cshaper']
+
             if 'tc' in ifstate:
                 self.tc[name] = TC(
                     name, ifstate['tc'])
@@ -241,7 +300,9 @@ class IfState():
                                 ipr.link('set', index=link.get(
                                     'index'), state='down')
                                 ipr.link('del', index=link.get('index'))
-                            except NetlinkError as err:
+                            except Exception as err:
+                                if not isinstance(err, netlinkerror_classes):
+                                    raise
                                 logger.warning('removing link {} failed: {}'.format(
                                     name, err.args[1]))
                     # shutdown physical interfaces
@@ -259,7 +320,9 @@ class IfState():
                                 try:
                                     ipr.link('set', index=link.get(
                                         'index'), state='down')
-                                except NetlinkError as err:
+                                except Exception as err:
+                                    if not isinstance(err, netlinkerror_classes):
+                                        raise
                                     logger.warning('updating link {} failed: {}'.format(
                                         name, err.args[1]))
             if not retry:
@@ -360,7 +423,8 @@ class IfState():
                             if k not in ['UNKNOWN', 'IFLA_VLAN_FLAGS']:
                                 attr = ipr_link.nla2name(k)
                                 if attr in Link.attr_value_maps:
-                                    ifs_link['link'][attr] = Link.attr_value_maps[attr].get(v, v)
+                                    ifs_link['link'][attr] = Link.attr_value_maps[attr].get(
+                                        v, v)
                                 else:
                                     ifs_link['link'][attr] = v
                 else:
@@ -384,7 +448,9 @@ class IfState():
                         try:
                             ifs_link['link'][attr] = ipr.get_ifname_by_index(
                                 ref)
-                        except NetlinkError as err:
+                        except Exception as err:
+                            if not isinstance(err, netlinkerror_classes):
+                                raise
                             logger.warning('lookup {} failed: {}'.format(
                                 attr, err.args[1]), extra={'iface': name})
                             ifs_link['link'][attr] = ref
